package ac.essex.ca;

import ac.essex.ca.util.GridPanel;
import ac.essex.ca.rules.RuleSet;

import javax.swing.*;
import java.awt.*;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * Demo Swing GUI to demonstrate how the Cellular Automata Program in Action
 * <p/>
 * <p/>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version,
 * provided that any use properly credits the author.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details at http://www.gnu.org
 * </p>
 *
 * @author Olly Oechsle, University of Essex
 * @version 1.0
 */
public class CADemo extends JFrame {

    RuleSet ruleSet;
    GridPanel mainPanel;

    /**
     * CA Parameters: size of Grid
     */
    int X = 200;
    int Y = 200;


    public static void main(String[] args) {
        new CADemo();
    }

    public CADemo() {

        /**
         * Create a rule set
         */
        ruleSet = new RuleSet(30);

        /**
         * Create a GraphPanel object and give it the function
         */
        mainPanel = new GridPanel(X,Y);

        // add the panel to the middle of the BorderLayout, it will fill the window.
        getContentPane().add(mainPanel, BorderLayout.CENTER);

        /**
         * Beneath is only standard SWING GUI stuff.
         */

        // Make sure Java Exits when the close button is clicked
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        }
        );

        final JTextField textField = new JTextField("1");
        textField.setPreferredSize(new Dimension(40, 25));
        final JButton button = new JButton("Go");
        final JLabel label = new JLabel("<html>Choose an id (1-256) and click 'Go' to display the cellular automata result.<br><small>VASE Lab Demo. http://vase.essex.ac.uk</html>");

        JPanel buttonBar = new JPanel(new FlowLayout(FlowLayout.LEFT));
        buttonBar.add(textField);
        buttonBar.add(button);
        buttonBar.add(label);

        getContentPane().add(buttonBar, BorderLayout.NORTH);

        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    int ruleSetID = Integer.parseInt(textField.getText());
                    iterateCA(ruleSetID);
                    ruleSetID++;
                    if (ruleSetID > 255) ruleSetID = 1;
                    textField.setText(String.valueOf(ruleSetID));
                } catch (NumberFormatException nfe) {
                    // do nothing - just a dodgy number
                }
            }
        });

        // default size of the window, the Graph Panel will be slightly smaller.
        setSize(640, 480);

        // window title
        setTitle("Cellular Automata Demo");

        // show the Window
        setVisible(true);

    }

    public void iterateCA(int ruleSetID) {

        ruleSet = new RuleSet(ruleSetID);

        mainPanel.clearData();

        /**
         * Initialise the grid with some data
         */
        mainPanel.setCell(100, 5, 1);

        int[][] data = ruleSet.evaluate(X,Y, mainPanel.getData());

        mainPanel.setData(data);

        mainPanel.repaint();

    }

}
